# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawFhirClient, RawFhirClient
from .types.fhir_bundle import FhirBundle
from .types.fhir_bundle_entry_item import FhirBundleEntryItem
from .types.fhir_patch_request_body_item import FhirPatchRequestBodyItem
from .types.fhir_resource import FhirResource
from .types.fhir_resource_meta import FhirResourceMeta
from .types.fhir_search_response import FhirSearchResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class FhirClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawFhirClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawFhirClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawFhirClient
        """
        return self._raw_client

    def search(
        self,
        fhir_provider_id: str,
        fhir_path: str,
        *,
        query_parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FhirSearchResponse:
        """
        Retrieves FHIR resources from the specified provider. Supports both individual resource retrieval and search operations based on the FHIR path and query parameters.

        The request is proxied to the configured FHIR server with appropriate authentication headers.

        Parameters
        ----------
        fhir_provider_id : str
            The ID of the FHIR provider to use. Can be either:
            - A UUID representing the provider ID
            - A provider name (legacy support - will just use the most recently updated provider with this name)

        fhir_path : str
            The FHIR resource path to operate on. This follows FHIR RESTful API conventions.
            Examples:
            - "Patient" (for resource type operations)
            - "Patient/123" (for specific resource operations)
            - "Patient/123/_history" (for history operations)

        query_parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            FHIR-compliant query parameters for search operations. Supports standard FHIR search parameters including:
            - Resource-specific search parameters (e.g., name for Patient, status for Observation)
            - Common search parameters (_id, _lastUpdated, _tag, _profile, _security, _text, _content, _filter)
            - Result parameters (_count, _offset, _sort, _include, _revinclude, _summary, _elements)
            - Search prefixes for dates, numbers, quantities (eq, ne, gt, ge, lt, le, sa, eb, ap)

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirSearchResponse
            Successfully retrieved FHIR resource(s)

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.fhir.search(
            fhir_provider_id="550e8400-e29b-41d4-a716-446655440000",
            fhir_path="Patient",
            phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
            phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
        )
        """
        _response = self._raw_client.search(
            fhir_provider_id,
            fhir_path,
            query_parameters=query_parameters,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            request_options=request_options,
        )
        return _response.data

    def create(
        self,
        fhir_provider_id: str,
        fhir_path: str,
        *,
        resource_type: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        id: typing.Optional[str] = OMIT,
        meta: typing.Optional[FhirResourceMeta] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FhirResource:
        """
        Creates a new FHIR resource on the specified provider. The request body should contain a valid FHIR resource in JSON format.

        The request is proxied to the configured FHIR server with appropriate authentication headers.

        Parameters
        ----------
        fhir_provider_id : str
            The ID of the FHIR provider to use. Can be either:
            - A UUID representing the provider ID
            - A provider name (legacy support - will just use the most recently updated provider with this name)

        fhir_path : str
            The FHIR resource path to operate on. This follows FHIR RESTful API conventions.
            Examples:
            - "Patient" (for resource type operations)
            - "Patient/123" (for specific resource operations)
            - "Patient/123/_history" (for history operations)

        resource_type : str
            The type of FHIR resource (e.g., Patient, Observation, etc.)

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        id : typing.Optional[str]
            Logical ID of the resource

        meta : typing.Optional[FhirResourceMeta]
            Metadata about the resource

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirResource
            Resource created successfully

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.fhir.create(
            fhir_provider_id="550e8400-e29b-41d4-a716-446655440000",
            fhir_path="Patient",
            phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
            phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
            resource_type="Patient",
        )
        """
        _response = self._raw_client.create(
            fhir_provider_id,
            fhir_path,
            resource_type=resource_type,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            id=id,
            meta=meta,
            request_options=request_options,
        )
        return _response.data

    def upsert(
        self,
        fhir_provider_id: str,
        fhir_path: str,
        *,
        resource_type: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        id: typing.Optional[str] = OMIT,
        meta: typing.Optional[FhirResourceMeta] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FhirResource:
        """
        Creates or updates a FHIR resource on the specified provider. If the resource exists, it will be updated; otherwise, it will be created.

        The request is proxied to the configured FHIR server with appropriate authentication headers.

        Parameters
        ----------
        fhir_provider_id : str
            The ID of the FHIR provider to use. Can be either:
            - A UUID representing the provider ID
            - A provider name (legacy support - will just use the most recently updated provider with this name)

        fhir_path : str
            The FHIR resource path to operate on. This follows FHIR RESTful API conventions.
            Examples:
            - "Patient" (for resource type operations)
            - "Patient/123" (for specific resource operations)
            - "Patient/123/_history" (for history operations)

        resource_type : str
            The type of FHIR resource (e.g., Patient, Observation, etc.)

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        id : typing.Optional[str]
            Logical ID of the resource

        meta : typing.Optional[FhirResourceMeta]
            Metadata about the resource

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirResource
            Resource upserted successfully

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.fhir.upsert(
            fhir_provider_id="550e8400-e29b-41d4-a716-446655440000",
            fhir_path="Patient",
            phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
            phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
            resource_type="Patient",
            id="123",
        )
        """
        _response = self._raw_client.upsert(
            fhir_provider_id,
            fhir_path,
            resource_type=resource_type,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            id=id,
            meta=meta,
            request_options=request_options,
        )
        return _response.data

    def delete(
        self,
        fhir_provider_id: str,
        fhir_path: str,
        *,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Deletes a FHIR resource from the specified provider.

        The request is proxied to the configured FHIR server with appropriate authentication headers.

        Parameters
        ----------
        fhir_provider_id : str
            The ID of the FHIR provider to use. Can be either:
            - A UUID representing the provider ID
            - A provider name (legacy support - will just use the most recently updated provider with this name)

        fhir_path : str
            The FHIR resource path to operate on. This follows FHIR RESTful API conventions.
            Examples:
            - "Patient" (for resource type operations)
            - "Patient/123" (for specific resource operations)
            - "Patient/123/_history" (for history operations)

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Resource deleted successfully

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.fhir.delete(
            fhir_provider_id="550e8400-e29b-41d4-a716-446655440000",
            fhir_path="Patient",
            phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
            phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
        )
        """
        _response = self._raw_client.delete(
            fhir_provider_id,
            fhir_path,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            request_options=request_options,
        )
        return _response.data

    def patch(
        self,
        fhir_provider_id: str,
        fhir_path: str,
        *,
        request: typing.Sequence[FhirPatchRequestBodyItem],
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FhirResource:
        """
        Partially updates a FHIR resource on the specified provider using JSON Patch operations as defined in RFC 6902.

        The request body should contain an array of JSON Patch operations. Each operation specifies:
        - `op`: The operation type (add, remove, replace, move, copy, test)
        - `path`: JSON Pointer to the target location in the resource
        - `value`: The value to use (required for add, replace, and test operations)

        The request is proxied to the configured FHIR server with appropriate authentication headers.

        Parameters
        ----------
        fhir_provider_id : str
            The ID of the FHIR provider to use. Can be either:
            - A UUID representing the provider ID
            - A provider name (legacy support - will just use the most recently updated provider with this name)

        fhir_path : str
            The FHIR resource path to operate on. This follows FHIR RESTful API conventions.
            Examples:
            - "Patient" (for resource type operations)
            - "Patient/123" (for specific resource operations)
            - "Patient/123/_history" (for history operations)

        request : typing.Sequence[FhirPatchRequestBodyItem]

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirResource
            Resource patched successfully

        Examples
        --------
        from phenoml import phenoml
        from phenoml.fhir import FhirPatchRequestBodyItem

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.fhir.patch(
            fhir_provider_id="550e8400-e29b-41d4-a716-446655440000",
            fhir_path="Patient",
            phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
            phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
            request=[
                FhirPatchRequestBodyItem(
                    op="replace",
                    path="/name/0/family",
                    value="NewFamilyName",
                )
            ],
        )
        """
        _response = self._raw_client.patch(
            fhir_provider_id,
            fhir_path,
            request=request,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            request_options=request_options,
        )
        return _response.data

    def execute_bundle(
        self,
        fhir_provider_id: str,
        *,
        entry: typing.Sequence[FhirBundleEntryItem],
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        total: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FhirBundle:
        """
        Executes a FHIR Bundle transaction or batch operation on the specified provider. This allows multiple FHIR resources to be processed in a single request.

        The request body should contain a valid FHIR Bundle resource with transaction or batch type.

        The request is proxied to the configured FHIR server with appropriate authentication headers.

        Parameters
        ----------
        fhir_provider_id : str
            The ID of the FHIR provider to use. Can be either:
            - A UUID representing the provider ID
            - A provider name (legacy support - will just use the most recently updated provider with this name)

        entry : typing.Sequence[FhirBundleEntryItem]
            Array of bundle entries containing resources or operation results

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        total : typing.Optional[int]
            Total number of resources that match the search criteria.
            Optional field as not all FHIR servers include it (e.g., Medplum).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirBundle
            Bundle executed successfully

        Examples
        --------
        from phenoml import phenoml
        from phenoml.fhir import FhirBundleEntryItem, FhirBundleEntryItemRequest

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.fhir.execute_bundle(
            fhir_provider_id="550e8400-e29b-41d4-a716-446655440000",
            phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
            phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
            entry=[
                FhirBundleEntryItem(
                    resource={
                        "resourceType": "Patient",
                        "name": [{"family": "Doe", "given": ["John"]}],
                    },
                    request=FhirBundleEntryItemRequest(
                        method="POST",
                        url="Patient",
                    ),
                ),
                FhirBundleEntryItem(
                    resource={
                        "resourceType": "Observation",
                        "status": "final",
                        "subject": {"reference": "Patient/123"},
                    },
                    request=FhirBundleEntryItemRequest(
                        method="POST",
                        url="Observation",
                    ),
                ),
            ],
        )
        """
        _response = self._raw_client.execute_bundle(
            fhir_provider_id,
            entry=entry,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            total=total,
            request_options=request_options,
        )
        return _response.data


class AsyncFhirClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawFhirClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawFhirClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawFhirClient
        """
        return self._raw_client

    async def search(
        self,
        fhir_provider_id: str,
        fhir_path: str,
        *,
        query_parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FhirSearchResponse:
        """
        Retrieves FHIR resources from the specified provider. Supports both individual resource retrieval and search operations based on the FHIR path and query parameters.

        The request is proxied to the configured FHIR server with appropriate authentication headers.

        Parameters
        ----------
        fhir_provider_id : str
            The ID of the FHIR provider to use. Can be either:
            - A UUID representing the provider ID
            - A provider name (legacy support - will just use the most recently updated provider with this name)

        fhir_path : str
            The FHIR resource path to operate on. This follows FHIR RESTful API conventions.
            Examples:
            - "Patient" (for resource type operations)
            - "Patient/123" (for specific resource operations)
            - "Patient/123/_history" (for history operations)

        query_parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            FHIR-compliant query parameters for search operations. Supports standard FHIR search parameters including:
            - Resource-specific search parameters (e.g., name for Patient, status for Observation)
            - Common search parameters (_id, _lastUpdated, _tag, _profile, _security, _text, _content, _filter)
            - Result parameters (_count, _offset, _sort, _include, _revinclude, _summary, _elements)
            - Search prefixes for dates, numbers, quantities (eq, ne, gt, ge, lt, le, sa, eb, ap)

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirSearchResponse
            Successfully retrieved FHIR resource(s)

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fhir.search(
                fhir_provider_id="550e8400-e29b-41d4-a716-446655440000",
                fhir_path="Patient",
                phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
                phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.search(
            fhir_provider_id,
            fhir_path,
            query_parameters=query_parameters,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            request_options=request_options,
        )
        return _response.data

    async def create(
        self,
        fhir_provider_id: str,
        fhir_path: str,
        *,
        resource_type: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        id: typing.Optional[str] = OMIT,
        meta: typing.Optional[FhirResourceMeta] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FhirResource:
        """
        Creates a new FHIR resource on the specified provider. The request body should contain a valid FHIR resource in JSON format.

        The request is proxied to the configured FHIR server with appropriate authentication headers.

        Parameters
        ----------
        fhir_provider_id : str
            The ID of the FHIR provider to use. Can be either:
            - A UUID representing the provider ID
            - A provider name (legacy support - will just use the most recently updated provider with this name)

        fhir_path : str
            The FHIR resource path to operate on. This follows FHIR RESTful API conventions.
            Examples:
            - "Patient" (for resource type operations)
            - "Patient/123" (for specific resource operations)
            - "Patient/123/_history" (for history operations)

        resource_type : str
            The type of FHIR resource (e.g., Patient, Observation, etc.)

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        id : typing.Optional[str]
            Logical ID of the resource

        meta : typing.Optional[FhirResourceMeta]
            Metadata about the resource

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirResource
            Resource created successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fhir.create(
                fhir_provider_id="550e8400-e29b-41d4-a716-446655440000",
                fhir_path="Patient",
                phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
                phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
                resource_type="Patient",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            fhir_provider_id,
            fhir_path,
            resource_type=resource_type,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            id=id,
            meta=meta,
            request_options=request_options,
        )
        return _response.data

    async def upsert(
        self,
        fhir_provider_id: str,
        fhir_path: str,
        *,
        resource_type: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        id: typing.Optional[str] = OMIT,
        meta: typing.Optional[FhirResourceMeta] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FhirResource:
        """
        Creates or updates a FHIR resource on the specified provider. If the resource exists, it will be updated; otherwise, it will be created.

        The request is proxied to the configured FHIR server with appropriate authentication headers.

        Parameters
        ----------
        fhir_provider_id : str
            The ID of the FHIR provider to use. Can be either:
            - A UUID representing the provider ID
            - A provider name (legacy support - will just use the most recently updated provider with this name)

        fhir_path : str
            The FHIR resource path to operate on. This follows FHIR RESTful API conventions.
            Examples:
            - "Patient" (for resource type operations)
            - "Patient/123" (for specific resource operations)
            - "Patient/123/_history" (for history operations)

        resource_type : str
            The type of FHIR resource (e.g., Patient, Observation, etc.)

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        id : typing.Optional[str]
            Logical ID of the resource

        meta : typing.Optional[FhirResourceMeta]
            Metadata about the resource

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirResource
            Resource upserted successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fhir.upsert(
                fhir_provider_id="550e8400-e29b-41d4-a716-446655440000",
                fhir_path="Patient",
                phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
                phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
                resource_type="Patient",
                id="123",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.upsert(
            fhir_provider_id,
            fhir_path,
            resource_type=resource_type,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            id=id,
            meta=meta,
            request_options=request_options,
        )
        return _response.data

    async def delete(
        self,
        fhir_provider_id: str,
        fhir_path: str,
        *,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Deletes a FHIR resource from the specified provider.

        The request is proxied to the configured FHIR server with appropriate authentication headers.

        Parameters
        ----------
        fhir_provider_id : str
            The ID of the FHIR provider to use. Can be either:
            - A UUID representing the provider ID
            - A provider name (legacy support - will just use the most recently updated provider with this name)

        fhir_path : str
            The FHIR resource path to operate on. This follows FHIR RESTful API conventions.
            Examples:
            - "Patient" (for resource type operations)
            - "Patient/123" (for specific resource operations)
            - "Patient/123/_history" (for history operations)

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Resource deleted successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fhir.delete(
                fhir_provider_id="550e8400-e29b-41d4-a716-446655440000",
                fhir_path="Patient",
                phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
                phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(
            fhir_provider_id,
            fhir_path,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            request_options=request_options,
        )
        return _response.data

    async def patch(
        self,
        fhir_provider_id: str,
        fhir_path: str,
        *,
        request: typing.Sequence[FhirPatchRequestBodyItem],
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FhirResource:
        """
        Partially updates a FHIR resource on the specified provider using JSON Patch operations as defined in RFC 6902.

        The request body should contain an array of JSON Patch operations. Each operation specifies:
        - `op`: The operation type (add, remove, replace, move, copy, test)
        - `path`: JSON Pointer to the target location in the resource
        - `value`: The value to use (required for add, replace, and test operations)

        The request is proxied to the configured FHIR server with appropriate authentication headers.

        Parameters
        ----------
        fhir_provider_id : str
            The ID of the FHIR provider to use. Can be either:
            - A UUID representing the provider ID
            - A provider name (legacy support - will just use the most recently updated provider with this name)

        fhir_path : str
            The FHIR resource path to operate on. This follows FHIR RESTful API conventions.
            Examples:
            - "Patient" (for resource type operations)
            - "Patient/123" (for specific resource operations)
            - "Patient/123/_history" (for history operations)

        request : typing.Sequence[FhirPatchRequestBodyItem]

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirResource
            Resource patched successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml
        from phenoml.fhir import FhirPatchRequestBodyItem

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fhir.patch(
                fhir_provider_id="550e8400-e29b-41d4-a716-446655440000",
                fhir_path="Patient",
                phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
                phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
                request=[
                    FhirPatchRequestBodyItem(
                        op="replace",
                        path="/name/0/family",
                        value="NewFamilyName",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.patch(
            fhir_provider_id,
            fhir_path,
            request=request,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            request_options=request_options,
        )
        return _response.data

    async def execute_bundle(
        self,
        fhir_provider_id: str,
        *,
        entry: typing.Sequence[FhirBundleEntryItem],
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        total: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FhirBundle:
        """
        Executes a FHIR Bundle transaction or batch operation on the specified provider. This allows multiple FHIR resources to be processed in a single request.

        The request body should contain a valid FHIR Bundle resource with transaction or batch type.

        The request is proxied to the configured FHIR server with appropriate authentication headers.

        Parameters
        ----------
        fhir_provider_id : str
            The ID of the FHIR provider to use. Can be either:
            - A UUID representing the provider ID
            - A provider name (legacy support - will just use the most recently updated provider with this name)

        entry : typing.Sequence[FhirBundleEntryItem]
            Array of bundle entries containing resources or operation results

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        total : typing.Optional[int]
            Total number of resources that match the search criteria.
            Optional field as not all FHIR servers include it (e.g., Medplum).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FhirBundle
            Bundle executed successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml
        from phenoml.fhir import FhirBundleEntryItem, FhirBundleEntryItemRequest

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fhir.execute_bundle(
                fhir_provider_id="550e8400-e29b-41d4-a716-446655440000",
                phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
                phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
                entry=[
                    FhirBundleEntryItem(
                        resource={
                            "resourceType": "Patient",
                            "name": [{"family": "Doe", "given": ["John"]}],
                        },
                        request=FhirBundleEntryItemRequest(
                            method="POST",
                            url="Patient",
                        ),
                    ),
                    FhirBundleEntryItem(
                        resource={
                            "resourceType": "Observation",
                            "status": "final",
                            "subject": {"reference": "Patient/123"},
                        },
                        request=FhirBundleEntryItemRequest(
                            method="POST",
                            url="Observation",
                        ),
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.execute_bundle(
            fhir_provider_id,
            entry=entry,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            total=total,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .step_operation import StepOperation
from .workflow_step_type import WorkflowStepType


class WorkflowStep(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the step
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Human-readable name for the step
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Detailed description of what the step does
    """

    type: typing.Optional[WorkflowStepType] = pydantic.Field(default=None)
    """
    Type of operation this step performs
    """

    operation: typing.Optional[StepOperation] = None
    provider_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Selected FHIR provider ID for this step
    """

    dependencies: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    IDs of steps this step depends on
    """

    dynamic_generation: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether to use dynamic lang2fhir generation for this step
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .fhir_provider_sandbox_info import FhirProviderSandboxInfo
from .fhir_provider_template import FhirProviderTemplate

FhirProviderListResponseFhirProvidersItem = typing.Union[FhirProviderTemplate, FhirProviderSandboxInfo]



# This file was auto-generated by Fern from our API Definition.

import typing

Provider = typing.Union[
    typing.Literal[
        "athenahealth", "canvas", "cerner", "elation", "epic", "google_healthcare", "hapi", "medplum", "sandbox"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .fhir_resource_meta import FhirResourceMeta


class FhirResource(UniversalBaseModel):
    """
    A generic FHIR resource. All FHIR resources share common elements like resourceType and id,
    but the specific structure depends on the resource type.
    """

    resource_type: typing_extensions.Annotated[str, FieldMetadata(alias="resourceType")] = pydantic.Field()
    """
    The type of FHIR resource (e.g., Patient, Observation, etc.)
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Logical ID of the resource
    """

    meta: typing.Optional[FhirResourceMeta] = pydantic.Field(default=None)
    """
    Metadata about the resource
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .bad_request_error_body import BadRequestErrorBody
from .unauthorized_error_body import UnauthorizedErrorBody

__all__ = ["BadRequestErrorBody", "UnauthorizedErrorBody"]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class McpServerToolCallResponse(UniversalBaseModel):
    success: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the MCP server tool was called successfully
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Status message
    """

    result: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Result of the MCP server tool call
    """

    error: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message if the call failed
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class Lang2FhirAndCreateMultiResponseResourceInfoItem(UniversalBaseModel):
    temp_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="tempId")] = pydantic.Field(
        default=None
    )
    """
    Original temporary UUID
    """

    resource_type: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="resourceType")] = (
        pydantic.Field(default=None)
    )
    """
    FHIR resource type
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Text excerpt this resource was extracted from
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

AuthMethod = typing.Union[
    typing.Literal["client_secret", "google_healthcare", "jwt", "on_behalf_of", "token_passthrough", "none"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .citation import Citation
from .code_response import CodeResponse
from .code_system_details import CodeSystemDetails
from .code_system_info import CodeSystemInfo
from .construe_upload_code_system_response import ConstrueUploadCodeSystemResponse
from .extract_codes_result import ExtractCodesResult
from .extract_request_config import ExtractRequestConfig
from .extract_request_config_chunking_method import ExtractRequestConfigChunkingMethod
from .extract_request_config_validation_method import ExtractRequestConfigValidationMethod
from .extract_request_system import ExtractRequestSystem
from .extracted_code_result import ExtractedCodeResult
from .get_code_response import GetCodeResponse
from .list_code_systems_response import ListCodeSystemsResponse
from .list_codes_response import ListCodesResponse
from .semantic_search_response import SemanticSearchResponse
from .semantic_search_result import SemanticSearchResult
from .text_search_response import TextSearchResponse
from .text_search_result import TextSearchResult
from .upload_request_format import UploadRequestFormat

__all__ = [
    "Citation",
    "CodeResponse",
    "CodeSystemDetails",
    "CodeSystemInfo",
    "ConstrueUploadCodeSystemResponse",
    "ExtractCodesResult",
    "ExtractRequestConfig",
    "ExtractRequestConfigChunkingMethod",
    "ExtractRequestConfigValidationMethod",
    "ExtractRequestSystem",
    "ExtractedCodeResult",
    "GetCodeResponse",
    "ListCodeSystemsResponse",
    "ListCodesResponse",
    "SemanticSearchResponse",
    "SemanticSearchResult",
    "TextSearchResponse",
    "TextSearchResult",
    "UploadRequestFormat",
]



# This file was auto-generated by Fern from our API Definition.

import typing

SearchResponseResourceType = typing.Union[
    typing.Literal[
        "AllergyIntolerance",
        "Appointment",
        "CarePlan",
        "CareTeam",
        "Condition",
        "Coverage",
        "Device",
        "DiagnosticReport",
        "DocumentReference",
        "Encounter",
        "Goal",
        "Immunization",
        "Location",
        "Medication",
        "MedicationRequest",
        "Observation",
        "Organization",
        "Patient",
        "PlanDefinition",
        "Practitioner",
        "PractitionerRole",
        "Procedure",
        "Provenance",
        "Questionnaire",
        "QuestionnaireResponse",
        "RelatedPerson",
        "Schedule",
        "ServiceRequest",
        "Slot",
        "Specimen",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

DocumentRequestFileType = typing.Union[
    typing.Literal["application/pdf", "image/png", "image/jpeg", "image/jpg"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    Citation,
    CodeResponse,
    CodeSystemDetails,
    CodeSystemInfo,
    ConstrueUploadCodeSystemResponse,
    ExtractCodesResult,
    ExtractRequestConfig,
    ExtractRequestConfigChunkingMethod,
    ExtractRequestConfigValidationMethod,
    ExtractRequestSystem,
    ExtractedCodeResult,
    GetCodeResponse,
    ListCodeSystemsResponse,
    ListCodesResponse,
    SemanticSearchResponse,
    SemanticSearchResult,
    TextSearchResponse,
    TextSearchResult,
    UploadRequestFormat,
)
from .errors import (
    BadRequestError,
    ConflictError,
    FailedDependencyError,
    InternalServerError,
    NotFoundError,
    NotImplementedError,
    ServiceUnavailableError,
    UnauthorizedError,
)

__all__ = [
    "BadRequestError",
    "Citation",
    "CodeResponse",
    "CodeSystemDetails",
    "CodeSystemInfo",
    "ConflictError",
    "ConstrueUploadCodeSystemResponse",
    "ExtractCodesResult",
    "ExtractRequestConfig",
    "ExtractRequestConfigChunkingMethod",
    "ExtractRequestConfigValidationMethod",
    "ExtractRequestSystem",
    "ExtractedCodeResult",
    "FailedDependencyError",
    "GetCodeResponse",
    "InternalServerError",
    "ListCodeSystemsResponse",
    "ListCodesResponse",
    "NotFoundError",
    "NotImplementedError",
    "SemanticSearchResponse",
    "SemanticSearchResult",
    "ServiceUnavailableError",
    "TextSearchResponse",
    "TextSearchResult",
    "UnauthorizedError",
    "UploadRequestFormat",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .agent_chat_response import AgentChatResponse
from .agent_create_request import AgentCreateRequest
from .agent_create_request_provider import AgentCreateRequestProvider
from .agent_delete_response import AgentDeleteResponse
from .agent_get_chat_messages_request_order import AgentGetChatMessagesRequestOrder
from .agent_get_chat_messages_request_role import AgentGetChatMessagesRequestRole
from .agent_get_chat_messages_response import AgentGetChatMessagesResponse
from .agent_list_response import AgentListResponse
from .agent_prompts_response import AgentPromptsResponse
from .agent_response import AgentResponse
from .agent_template import AgentTemplate
from .agent_template_provider import AgentTemplateProvider
from .chat_message_template import ChatMessageTemplate
from .chat_message_template_role import ChatMessageTemplateRole
from .chat_session_template import ChatSessionTemplate
from .json_patch import JsonPatch
from .json_patch_operation import JsonPatchOperation
from .json_patch_operation_op import JsonPatchOperationOp
from .prompt_template import PromptTemplate
from .success_response import SuccessResponse

__all__ = [
    "AgentChatResponse",
    "AgentCreateRequest",
    "AgentCreateRequestProvider",
    "AgentDeleteResponse",
    "AgentGetChatMessagesRequestOrder",
    "AgentGetChatMessagesRequestRole",
    "AgentGetChatMessagesResponse",
    "AgentListResponse",
    "AgentPromptsResponse",
    "AgentResponse",
    "AgentTemplate",
    "AgentTemplateProvider",
    "ChatMessageTemplate",
    "ChatMessageTemplateRole",
    "ChatSessionTemplate",
    "JsonPatch",
    "JsonPatchOperation",
    "JsonPatchOperationOp",
    "PromptTemplate",
    "SuccessResponse",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from .errors.bad_request_error import BadRequestError
from .errors.failed_dependency_error import FailedDependencyError
from .errors.forbidden_error import ForbiddenError
from .errors.internal_server_error import InternalServerError
from .errors.unauthorized_error import UnauthorizedError
from .types.cohort_response import CohortResponse
from .types.lang2fhir_and_create_multi_response import Lang2FhirAndCreateMultiResponse
from .types.lang2fhir_and_create_request_resource import Lang2FhirAndCreateRequestResource
from .types.lang2fhir_and_create_response import Lang2FhirAndCreateResponse
from .types.lang2fhir_and_search_response import Lang2FhirAndSearchResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawToolsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_fhir_resource(
        self,
        *,
        resource: Lang2FhirAndCreateRequestResource,
        text: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        provider: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Lang2FhirAndCreateResponse]:
        """
        Converts natural language to FHIR resource and optionally stores it in a FHIR server

        Parameters
        ----------
        resource : Lang2FhirAndCreateRequestResource
            Type of FHIR resource to create. Use 'auto' for automatic resource type detection, or specify a supported US Core profile.

        text : str
            Natural language text to convert to FHIR resource

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        provider : typing.Optional[str]
            FHIR provider ID - must be a valid UUID from existing FHIR providers. also supports provider by name (e.g. medplum)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Lang2FhirAndCreateResponse]
            Successfully created FHIR resource
        """
        _response = self._client_wrapper.httpx_client.request(
            "tools/lang2fhir-and-create",
            method="POST",
            json={
                "resource": resource,
                "text": text,
                "provider": provider,
            },
            headers={
                "content-type": "application/json",
                "X-Phenoml-On-Behalf-Of": str(phenoml_on_behalf_of) if phenoml_on_behalf_of is not None else None,
                "X-Phenoml-Fhir-Provider": str(phenoml_fhir_provider) if phenoml_fhir_provider is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Lang2FhirAndCreateResponse,
                    parse_obj_as(
                        type_=Lang2FhirAndCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_fhir_resources_multi(
        self,
        *,
        text: str,
        provider: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Lang2FhirAndCreateMultiResponse]:
        """
        Extracts multiple FHIR resources from natural language text and stores them in a FHIR server.
        Automatically detects Patient, Condition, MedicationRequest, Observation, and other resource types.
        Resources are linked with proper references and submitted as a transaction bundle.
        For FHIR servers that don't auto-resolve urn:uuid references, this endpoint will automatically
        resolve them via PUT requests after the initial bundle creation.

        Parameters
        ----------
        text : str
            Natural language text containing multiple clinical concepts to extract

        provider : str
            FHIR provider ID or name

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        version : typing.Optional[str]
            FHIR version to use

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Lang2FhirAndCreateMultiResponse]
            Successfully created FHIR resources
        """
        _response = self._client_wrapper.httpx_client.request(
            "tools/lang2fhir-and-create-multi",
            method="POST",
            json={
                "text": text,
                "version": version,
                "provider": provider,
            },
            headers={
                "content-type": "application/json",
                "X-Phenoml-On-Behalf-Of": str(phenoml_on_behalf_of) if phenoml_on_behalf_of is not None else None,
                "X-Phenoml-Fhir-Provider": str(phenoml_fhir_provider) if phenoml_fhir_provider is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Lang2FhirAndCreateMultiResponse,
                    parse_obj_as(
                        type_=Lang2FhirAndCreateMultiResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def search_fhir_resources(
        self,
        *,
        text: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        patient_id: typing.Optional[str] = OMIT,
        practitioner_id: typing.Optional[str] = OMIT,
        count: typing.Optional[int] = OMIT,
        provider: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Lang2FhirAndSearchResponse]:
        """
        Converts natural language to FHIR search parameters and executes search in FHIR server

        Parameters
        ----------
        text : str
            Natural language text to convert to FHIR search parameters

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        patient_id : typing.Optional[str]
            Patient ID to filter results

        practitioner_id : typing.Optional[str]
            Practitioner ID to filter results

        count : typing.Optional[int]
            Maximum number of results to return

        provider : typing.Optional[str]
            FHIR provider ID - must be a valid UUID from existing FHIR providers. also supports provider by name (e.g. medplum)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Lang2FhirAndSearchResponse]
            Successfully generated search and retrieved results
        """
        _response = self._client_wrapper.httpx_client.request(
            "tools/lang2fhir-and-search",
            method="POST",
            json={
                "text": text,
                "patient_id": patient_id,
                "practitioner_id": practitioner_id,
                "count": count,
                "provider": provider,
            },
            headers={
                "content-type": "application/json",
                "X-Phenoml-On-Behalf-Of": str(phenoml_on_behalf_of) if phenoml_on_behalf_of is not None else None,
                "X-Phenoml-Fhir-Provider": str(phenoml_fhir_provider) if phenoml_fhir_provider is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Lang2FhirAndSearchResponse,
                    parse_obj_as(
                        type_=Lang2FhirAndSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def analyze_cohort(
        self,
        *,
        text: str,
        provider: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CohortResponse]:
        """
        Uses LLM to extract search concepts from natural language and builds patient cohorts with inclusion/exclusion criteria

        Parameters
        ----------
        text : str
            Natural language text describing the patient cohort criteria

        provider : str
            FHIR provider ID - must be a valid UUID from existing FHIR providers. also supports provider by name (e.g. medplum)

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CohortResponse]
            Successfully analyzed cohort and retrieved patient list
        """
        _response = self._client_wrapper.httpx_client.request(
            "tools/cohort",
            method="POST",
            json={
                "text": text,
                "provider": provider,
            },
            headers={
                "content-type": "application/json",
                "X-Phenoml-On-Behalf-Of": str(phenoml_on_behalf_of) if phenoml_on_behalf_of is not None else None,
                "X-Phenoml-Fhir-Provider": str(phenoml_fhir_provider) if phenoml_fhir_provider is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CohortResponse,
                    parse_obj_as(
                        type_=CohortResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawToolsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_fhir_resource(
        self,
        *,
        resource: Lang2FhirAndCreateRequestResource,
        text: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        provider: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Lang2FhirAndCreateResponse]:
        """
        Converts natural language to FHIR resource and optionally stores it in a FHIR server

        Parameters
        ----------
        resource : Lang2FhirAndCreateRequestResource
            Type of FHIR resource to create. Use 'auto' for automatic resource type detection, or specify a supported US Core profile.

        text : str
            Natural language text to convert to FHIR resource

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        provider : typing.Optional[str]
            FHIR provider ID - must be a valid UUID from existing FHIR providers. also supports provider by name (e.g. medplum)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Lang2FhirAndCreateResponse]
            Successfully created FHIR resource
        """
        _response = await self._client_wrapper.httpx_client.request(
            "tools/lang2fhir-and-create",
            method="POST",
            json={
                "resource": resource,
                "text": text,
                "provider": provider,
            },
            headers={
                "content-type": "application/json",
                "X-Phenoml-On-Behalf-Of": str(phenoml_on_behalf_of) if phenoml_on_behalf_of is not None else None,
                "X-Phenoml-Fhir-Provider": str(phenoml_fhir_provider) if phenoml_fhir_provider is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Lang2FhirAndCreateResponse,
                    parse_obj_as(
                        type_=Lang2FhirAndCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_fhir_resources_multi(
        self,
        *,
        text: str,
        provider: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Lang2FhirAndCreateMultiResponse]:
        """
        Extracts multiple FHIR resources from natural language text and stores them in a FHIR server.
        Automatically detects Patient, Condition, MedicationRequest, Observation, and other resource types.
        Resources are linked with proper references and submitted as a transaction bundle.
        For FHIR servers that don't auto-resolve urn:uuid references, this endpoint will automatically
        resolve them via PUT requests after the initial bundle creation.

        Parameters
        ----------
        text : str
            Natural language text containing multiple clinical concepts to extract

        provider : str
            FHIR provider ID or name

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        version : typing.Optional[str]
            FHIR version to use

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Lang2FhirAndCreateMultiResponse]
            Successfully created FHIR resources
        """
        _response = await self._client_wrapper.httpx_client.request(
            "tools/lang2fhir-and-create-multi",
            method="POST",
            json={
                "text": text,
                "version": version,
                "provider": provider,
            },
            headers={
                "content-type": "application/json",
                "X-Phenoml-On-Behalf-Of": str(phenoml_on_behalf_of) if phenoml_on_behalf_of is not None else None,
                "X-Phenoml-Fhir-Provider": str(phenoml_fhir_provider) if phenoml_fhir_provider is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Lang2FhirAndCreateMultiResponse,
                    parse_obj_as(
                        type_=Lang2FhirAndCreateMultiResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def search_fhir_resources(
        self,
        *,
        text: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        patient_id: typing.Optional[str] = OMIT,
        practitioner_id: typing.Optional[str] = OMIT,
        count: typing.Optional[int] = OMIT,
        provider: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Lang2FhirAndSearchResponse]:
        """
        Converts natural language to FHIR search parameters and executes search in FHIR server

        Parameters
        ----------
        text : str
            Natural language text to convert to FHIR search parameters

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        patient_id : typing.Optional[str]
            Patient ID to filter results

        practitioner_id : typing.Optional[str]
            Practitioner ID to filter results

        count : typing.Optional[int]
            Maximum number of results to return

        provider : typing.Optional[str]
            FHIR provider ID - must be a valid UUID from existing FHIR providers. also supports provider by name (e.g. medplum)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Lang2FhirAndSearchResponse]
            Successfully generated search and retrieved results
        """
        _response = await self._client_wrapper.httpx_client.request(
            "tools/lang2fhir-and-search",
            method="POST",
            json={
                "text": text,
                "patient_id": patient_id,
                "practitioner_id": practitioner_id,
                "count": count,
                "provider": provider,
            },
            headers={
                "content-type": "application/json",
                "X-Phenoml-On-Behalf-Of": str(phenoml_on_behalf_of) if phenoml_on_behalf_of is not None else None,
                "X-Phenoml-Fhir-Provider": str(phenoml_fhir_provider) if phenoml_fhir_provider is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Lang2FhirAndSearchResponse,
                    parse_obj_as(
                        type_=Lang2FhirAndSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def analyze_cohort(
        self,
        *,
        text: str,
        provider: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CohortResponse]:
        """
        Uses LLM to extract search concepts from natural language and builds patient cohorts with inclusion/exclusion criteria

        Parameters
        ----------
        text : str
            Natural language text describing the patient cohort criteria

        provider : str
            FHIR provider ID - must be a valid UUID from existing FHIR providers. also supports provider by name (e.g. medplum)

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CohortResponse]
            Successfully analyzed cohort and retrieved patient list
        """
        _response = await self._client_wrapper.httpx_client.request(
            "tools/cohort",
            method="POST",
            json={
                "text": text,
                "provider": provider,
            },
            headers={
                "content-type": "application/json",
                "X-Phenoml-On-Behalf-Of": str(phenoml_on_behalf_of) if phenoml_on_behalf_of is not None else None,
                "X-Phenoml-Fhir-Provider": str(phenoml_fhir_provider) if phenoml_fhir_provider is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CohortResponse,
                    parse_obj_as(
                        type_=CohortResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from .errors.bad_request_error import BadRequestError
from .errors.forbidden_error import ForbiddenError
from .errors.internal_server_error import InternalServerError
from .errors.not_found_error import NotFoundError
from .errors.unauthorized_error import UnauthorizedError
from .types.auth_method import AuthMethod
from .types.fhir_provider_delete_response import FhirProviderDeleteResponse
from .types.fhir_provider_list_response import FhirProviderListResponse
from .types.fhir_provider_remove_auth_config_response import FhirProviderRemoveAuthConfigResponse
from .types.fhir_provider_response import FhirProviderResponse
from .types.provider import Provider
from .types.role import Role
from .types.service_account_key import ServiceAccountKey

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawFhirProviderClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        name: str,
        provider: Provider,
        auth_method: AuthMethod,
        base_url: str,
        description: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        client_secret: typing.Optional[str] = OMIT,
        service_account_key: typing.Optional[ServiceAccountKey] = OMIT,
        role: typing.Optional[Role] = OMIT,
        scopes: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[FhirProviderResponse]:
        """
        Creates a new FHIR provider configuration with authentication credentials.

        Note: The "sandbox" provider type cannot be created via this API - it is managed internally.

        Parameters
        ----------
        name : str
            Display name for the FHIR provider

        provider : Provider

        auth_method : AuthMethod

        base_url : str
            Base URL of the FHIR server

        description : typing.Optional[str]
            Optional description of the FHIR provider

        client_id : typing.Optional[str]
            OAuth client ID (required for most auth methods)

        client_secret : typing.Optional[str]
            OAuth client secret (required for client_secret and on_behalf_of auth methods)

        service_account_key : typing.Optional[ServiceAccountKey]

        role : typing.Optional[Role]

        scopes : typing.Optional[str]
            OAuth scopes to request. Cannot be specified with role. If neither role nor scopes are specified, the provider-specific default role will be used. You are solely responsible for ensuring the scopes are valid options for the provider being created or updated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[FhirProviderResponse]
            FHIR provider created successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            "fhir-provider",
            method="POST",
            json={
                "name": name,
                "description": description,
                "provider": provider,
                "auth_method": auth_method,
                "base_url": base_url,
                "client_id": client_id,
                "client_secret": client_secret,
                "service_account_key": convert_and_respect_annotation_metadata(
                    object_=service_account_key, annotation=ServiceAccountKey, direction="write"
                ),
                "role": role,
                "scopes": scopes,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirProviderResponse,
                    parse_obj_as(
                        type_=FhirProviderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[FhirProviderListResponse]:
        """
        Retrieves a list of all active FHIR providers for the authenticated user.

        On shared instances, only sandbox providers are returned.
        Sandbox providers return FhirProviderSandboxInfo.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[FhirProviderListResponse]
            FHIR providers retrieved successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            "fhir-provider/list",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirProviderListResponse,
                    parse_obj_as(
                        type_=FhirProviderListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, fhir_provider_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[FhirProviderResponse]:
        """
        Retrieves a specific FHIR provider configuration by its ID.

        Sandbox providers return FhirProviderSandboxInfo.
        On shared instances, only sandbox providers can be accessed.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[FhirProviderResponse]
            FHIR provider retrieved successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"fhir-provider/{jsonable_encoder(fhir_provider_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirProviderResponse,
                    parse_obj_as(
                        type_=FhirProviderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, fhir_provider_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[FhirProviderDeleteResponse]:
        """
        Soft deletes a FHIR provider by setting is_active to false.

        Note: Sandbox providers cannot be deleted.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[FhirProviderDeleteResponse]
            FHIR provider deleted successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"fhir-provider/{jsonable_encoder(fhir_provider_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirProviderDeleteResponse,
                    parse_obj_as(
                        type_=FhirProviderDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def add_auth_config(
        self,
        fhir_provider_id: str,
        *,
        auth_method: AuthMethod,
        client_secret: typing.Optional[str] = OMIT,
        service_account_key: typing.Optional[ServiceAccountKey] = OMIT,
        credential_expiry: typing.Optional[dt.datetime] = OMIT,
        role: typing.Optional[Role] = OMIT,
        scopes: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[FhirProviderResponse]:
        """
        Adds a new authentication configuration to an existing FHIR provider.
        This enables key rotation and multiple auth configurations per provider.

        Note: Sandbox providers cannot be modified.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider to add auth config to

        auth_method : AuthMethod

        client_secret : typing.Optional[str]
            OAuth client secret (required for client_secret and on_behalf_of auth methods)

        service_account_key : typing.Optional[ServiceAccountKey]

        credential_expiry : typing.Optional[dt.datetime]
            Expiry time for JWT credentials (only applicable for JWT auth method)

        role : typing.Optional[Role]

        scopes : typing.Optional[str]
            OAuth scopes to request. Cannot be specified with role. If neither role nor scopes are specified, the provider-specific default role will be used. You are solely responsible for ensuring the scopes are valid options for the provider being created or updated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[FhirProviderResponse]
            Auth configuration added successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"fhir-provider/{jsonable_encoder(fhir_provider_id)}/add-auth-config",
            method="PATCH",
            json={
                "auth_method": auth_method,
                "client_secret": client_secret,
                "service_account_key": convert_and_respect_annotation_metadata(
                    object_=service_account_key, annotation=ServiceAccountKey, direction="write"
                ),
                "credential_expiry": credential_expiry,
                "role": role,
                "scopes": scopes,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirProviderResponse,
                    parse_obj_as(
                        type_=FhirProviderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def set_active_auth_config(
        self, fhir_provider_id: str, *, auth_config_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[FhirProviderResponse]:
        """
        Sets which authentication configuration should be active for a FHIR provider.
        Only one auth config can be active at a time.

        If the specified auth config is already active, the request succeeds without
        making any changes and returns a message indicating the config is already active.

        Note: Sandbox providers cannot be modified.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider

        auth_config_id : str
            ID of the auth configuration to set as active

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[FhirProviderResponse]
            Active auth configuration set successfully, or the config was already active.
            Check the message field to determine which case occurred.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"fhir-provider/{jsonable_encoder(fhir_provider_id)}/set-active-auth-config",
            method="PATCH",
            json={
                "auth_config_id": auth_config_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirProviderResponse,
                    parse_obj_as(
                        type_=FhirProviderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def remove_auth_config(
        self, fhir_provider_id: str, *, auth_config_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[FhirProviderRemoveAuthConfigResponse]:
        """
        Removes an authentication configuration from a FHIR provider.
        Cannot remove the currently active auth configuration.

        Note: Sandbox providers cannot be modified.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider

        auth_config_id : str
            ID of the auth configuration to remove

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[FhirProviderRemoveAuthConfigResponse]
            Auth configuration removed successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"fhir-provider/{jsonable_encoder(fhir_provider_id)}/remove-auth-config",
            method="PATCH",
            json={
                "auth_config_id": auth_config_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirProviderRemoveAuthConfigResponse,
                    parse_obj_as(
                        type_=FhirProviderRemoveAuthConfigResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawFhirProviderClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        name: str,
        provider: Provider,
        auth_method: AuthMethod,
        base_url: str,
        description: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        client_secret: typing.Optional[str] = OMIT,
        service_account_key: typing.Optional[ServiceAccountKey] = OMIT,
        role: typing.Optional[Role] = OMIT,
        scopes: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[FhirProviderResponse]:
        """
        Creates a new FHIR provider configuration with authentication credentials.

        Note: The "sandbox" provider type cannot be created via this API - it is managed internally.

        Parameters
        ----------
        name : str
            Display name for the FHIR provider

        provider : Provider

        auth_method : AuthMethod

        base_url : str
            Base URL of the FHIR server

        description : typing.Optional[str]
            Optional description of the FHIR provider

        client_id : typing.Optional[str]
            OAuth client ID (required for most auth methods)

        client_secret : typing.Optional[str]
            OAuth client secret (required for client_secret and on_behalf_of auth methods)

        service_account_key : typing.Optional[ServiceAccountKey]

        role : typing.Optional[Role]

        scopes : typing.Optional[str]
            OAuth scopes to request. Cannot be specified with role. If neither role nor scopes are specified, the provider-specific default role will be used. You are solely responsible for ensuring the scopes are valid options for the provider being created or updated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[FhirProviderResponse]
            FHIR provider created successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fhir-provider",
            method="POST",
            json={
                "name": name,
                "description": description,
                "provider": provider,
                "auth_method": auth_method,
                "base_url": base_url,
                "client_id": client_id,
                "client_secret": client_secret,
                "service_account_key": convert_and_respect_annotation_metadata(
                    object_=service_account_key, annotation=ServiceAccountKey, direction="write"
                ),
                "role": role,
                "scopes": scopes,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirProviderResponse,
                    parse_obj_as(
                        type_=FhirProviderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[FhirProviderListResponse]:
        """
        Retrieves a list of all active FHIR providers for the authenticated user.

        On shared instances, only sandbox providers are returned.
        Sandbox providers return FhirProviderSandboxInfo.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[FhirProviderListResponse]
            FHIR providers retrieved successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fhir-provider/list",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirProviderListResponse,
                    parse_obj_as(
                        type_=FhirProviderListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, fhir_provider_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[FhirProviderResponse]:
        """
        Retrieves a specific FHIR provider configuration by its ID.

        Sandbox providers return FhirProviderSandboxInfo.
        On shared instances, only sandbox providers can be accessed.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[FhirProviderResponse]
            FHIR provider retrieved successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"fhir-provider/{jsonable_encoder(fhir_provider_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirProviderResponse,
                    parse_obj_as(
                        type_=FhirProviderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, fhir_provider_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[FhirProviderDeleteResponse]:
        """
        Soft deletes a FHIR provider by setting is_active to false.

        Note: Sandbox providers cannot be deleted.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[FhirProviderDeleteResponse]
            FHIR provider deleted successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"fhir-provider/{jsonable_encoder(fhir_provider_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirProviderDeleteResponse,
                    parse_obj_as(
                        type_=FhirProviderDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def add_auth_config(
        self,
        fhir_provider_id: str,
        *,
        auth_method: AuthMethod,
        client_secret: typing.Optional[str] = OMIT,
        service_account_key: typing.Optional[ServiceAccountKey] = OMIT,
        credential_expiry: typing.Optional[dt.datetime] = OMIT,
        role: typing.Optional[Role] = OMIT,
        scopes: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[FhirProviderResponse]:
        """
        Adds a new authentication configuration to an existing FHIR provider.
        This enables key rotation and multiple auth configurations per provider.

        Note: Sandbox providers cannot be modified.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider to add auth config to

        auth_method : AuthMethod

        client_secret : typing.Optional[str]
            OAuth client secret (required for client_secret and on_behalf_of auth methods)

        service_account_key : typing.Optional[ServiceAccountKey]

        credential_expiry : typing.Optional[dt.datetime]
            Expiry time for JWT credentials (only applicable for JWT auth method)

        role : typing.Optional[Role]

        scopes : typing.Optional[str]
            OAuth scopes to request. Cannot be specified with role. If neither role nor scopes are specified, the provider-specific default role will be used. You are solely responsible for ensuring the scopes are valid options for the provider being created or updated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[FhirProviderResponse]
            Auth configuration added successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"fhir-provider/{jsonable_encoder(fhir_provider_id)}/add-auth-config",
            method="PATCH",
            json={
                "auth_method": auth_method,
                "client_secret": client_secret,
                "service_account_key": convert_and_respect_annotation_metadata(
                    object_=service_account_key, annotation=ServiceAccountKey, direction="write"
                ),
                "credential_expiry": credential_expiry,
                "role": role,
                "scopes": scopes,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirProviderResponse,
                    parse_obj_as(
                        type_=FhirProviderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def set_active_auth_config(
        self, fhir_provider_id: str, *, auth_config_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[FhirProviderResponse]:
        """
        Sets which authentication configuration should be active for a FHIR provider.
        Only one auth config can be active at a time.

        If the specified auth config is already active, the request succeeds without
        making any changes and returns a message indicating the config is already active.

        Note: Sandbox providers cannot be modified.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider

        auth_config_id : str
            ID of the auth configuration to set as active

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[FhirProviderResponse]
            Active auth configuration set successfully, or the config was already active.
            Check the message field to determine which case occurred.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"fhir-provider/{jsonable_encoder(fhir_provider_id)}/set-active-auth-config",
            method="PATCH",
            json={
                "auth_config_id": auth_config_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirProviderResponse,
                    parse_obj_as(
                        type_=FhirProviderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def remove_auth_config(
        self, fhir_provider_id: str, *, auth_config_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[FhirProviderRemoveAuthConfigResponse]:
        """
        Removes an authentication configuration from a FHIR provider.
        Cannot remove the currently active auth configuration.

        Note: Sandbox providers cannot be modified.

        Parameters
        ----------
        fhir_provider_id : str
            ID of the FHIR provider

        auth_config_id : str
            ID of the auth configuration to remove

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[FhirProviderRemoveAuthConfigResponse]
            Auth configuration removed successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"fhir-provider/{jsonable_encoder(fhir_provider_id)}/remove-auth-config",
            method="PATCH",
            json={
                "auth_config_id": auth_config_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FhirProviderRemoveAuthConfigResponse,
                    parse_obj_as(
                        type_=FhirProviderRemoveAuthConfigResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .code_system_info import CodeSystemInfo
from .semantic_search_result import SemanticSearchResult


class SemanticSearchResponse(UniversalBaseModel):
    system: CodeSystemInfo
    results: typing.List[SemanticSearchResult] = pydantic.Field()
    """
    Codes ordered by semantic similarity (most similar first)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .code_system_info import CodeSystemInfo
from .text_search_result import TextSearchResult


class TextSearchResponse(UniversalBaseModel):
    system: CodeSystemInfo
    results: typing.List[TextSearchResult] = pydantic.Field()
    """
    Codes matching the search query
    """

    found: int = pydantic.Field()
    """
    Total number of matching results (may exceed results array due to pagination)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ExecuteWorkflowResponseResults(UniversalBaseModel):
    steps: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Results for each executed workflow step, keyed by step ID
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
